/*************************************************************************/ /*!
@File
@Title          Services Debug Info interface.
@Copyright      Copyright (c) Imagination Technologies Ltd. All Rights Reserved
@Description    Defines the client side interface for Debug Info Framework.
@License        Strictly Confidential.
*/ /**************************************************************************/

#ifndef PVRSRV_DI_H
#define PVRSRV_DI_H

#include "pvrsrv_error.h"
#include "services.h"

typedef struct DI_CONTEXT_TAG DI_CONTEXT;
typedef void (*PFN_PROCESS_PACKET)(void *pvPriv, const IMG_BYTE *pbData,
                                   IMG_UINT32 ui32Size);

/*! @Function PVRSRVDICreateContext
 *
 * @Description
 * Creates a Debug Info context that is required for reading and writing
 * entries.
 *
 * @Input psConnection: Services connection handle
 *
 * @Output phContext: handle to the DI context
 *
 * @Return PVRSRV_ERROR error code, mainly:
 *         PVRSRV_ERROR_INVALID_PARAMS
 *         PVRSRV_ERROR_OUT_OF_MEMORY
 *         error codes returned by PVRSRVTLOpenStream()
 */
PVRSRV_ERROR PVRSRVDICreateContext(PVRSRV_DEV_CONNECTION *psConnection,
                                   IMG_HANDLE *phContext);

/*! @Function PVRSRVDIDestroyContext
 *
 * @Description
 * Destroys context created by PVRSRVDICreateContext()
 *
 * @Input psConnection: Services connection handle
 *
 * @Output psContext: handle to the DI context
 *
 * @Return PVRSRV_ERROR error code, mainly:
 *         PVRSRV_ERROR_INVALID_PARAMS
 *         PVRSRV_ERROR_OUT_OF_MEMORY
 *         error codes returned by PVRSRVTLOpenStream()
 */
PVRSRV_ERROR PVRSRVDIDestroyContext(PVRSRV_DEV_CONNECTION *psConnection,
                                    DI_CONTEXT *psContext);

/*! @Function PVRSRVDIReadSequentialEntry
 *
 * @Description
 * Reads data from an entry under pszEntryPath. Each time a packet is read
 * from the entry pfnProcessPacket callback will be called with a pointer
 * to the packet data and the data size as well as with the pvPriv pointer.
 *
 * Packet are usually null terminated string possibly orginised in multiple
 * lines where the new line character is determined by the '\n' character.
 *
 * @Input psConnection: Services connection handle
 * @Input psContext: handle to the DI context
 * @Input pszEntryPath: null-terminated entry path
 * @input pfnProcessPacket: callback function called every time a packet is
 *                          successfully read
 * @Inout pvPriv: private data passed to the pfnProcessPacket callback
 *
 * @Return PVRSRV_ERROR error code, mainly:
 *         error coded returned by
 *           PVRSRVTLAcquireData()
 *           PVRSRVTLReleaseData()
 */
PVRSRV_ERROR PVRSRVDIReadSequentialEntry(PVRSRV_DEV_CONNECTION *psConnection,
                                         DI_CONTEXT *psContext,
                                         const IMG_CHAR *pszEntryPath,
                                         PFN_PROCESS_PACKET pfnProcessPacket,
                                         void *pvPriv);

/*! @Function PVRSRVDIWriteEntry
 *
 * @Description
 * Writes pszValue to an entry under pszEntryPath.
 *
 * Packet are usually null terminated string possibly orginised in multiple
 * lines where the new line character is determined by the '\n' character.
 *
 * @Input psConnection: Services connection handle
 * @Input psContext: handle to the DI context
 * @Input pszEntryPath: null-terminated entry path
 * @input pszValue: null-terminated value
 *
 * @Return PVRSRV_ERROR error code
 */
PVRSRV_ERROR PVRSRVDIWriteEntry(PVRSRV_DEV_CONNECTION *psConnection,
                                DI_CONTEXT *psContext,
                                const IMG_CHAR *pszEntryPath,
                                const IMG_CHAR *pszValue);

/*! @Function PVRSRVDIReadRandomAccessEntry
 *
 * @Description
 * Reads data from an entry under pszEntryPath at a given ui64Offset offset
 * and of ui64Size size.
 *
 * Each time a packet is read from the entry pfnProcessPacket callback will
 * be called with a pointer to the packet data and the data size as well as
 * with the pvPriv pointer.
 *
 * The data passed to the callback might be smaller than requested size
 * but if enough data is available in the entry it should add up to the
 * desired sized during multiple calls of the callback functions.
 *
 * The internal format of the data is dependent on the particular entry.
 *
 * @Input psConnection: Services connection handle
 * @Input psContext: handle to the DI context
 * @Input pszEntryPath: null-terminated entry path
 * @Input ui64Offset: offset of the data withing the entry
 * @Input ui64Size: size of the data ot be read
 * @input pfnProcessPacket: callback function called every time a packet is
 *                          successfully read
 * @Inout pvPriv: private data passed to the pfnProcessPacket callback
 *
 * @Return PVRSRV_ERROR error code, mainly:
 *         error coded returned by
 *           PVRSRVTLAcquireData()
 *           PVRSRVTLReleaseData()
 */
PVRSRV_ERROR PVRSRVDIReadRandomAccessEntry(PVRSRV_DEV_CONNECTION *psConnection,
                                           DI_CONTEXT *psContext,
                                           const IMG_CHAR *pszEntryPath,
                                           IMG_UINT64 ui64Offset,
                                           IMG_UINT64 ui64Size,
                                           PFN_PROCESS_PACKET pfnProcessPacket,
                                           void *pvPriv);

/*! @Function PVRSRVDIListAllEntries
 *
 * @Description
 * List all Debug Info entries in the global DI entry tree. Each time a packet is read
 * from the entry pfnProcessPacket callback will be called with a pointer
 * to the packet data and the data size as well as with the pvPriv pointer.
 *
 * Packet are usually null terminated string possibly orginised in multiple
 * lines where the new line character is determined by the '\n' character.
 *
 * @Input psConnection: Services connection handle
 * @Input psContext: handle to the DI context
 * @input pfnProcessPacket: callback function called every time a packet is
 *                          successfully read
 * @Inout pvPriv: private data passed to the pfnProcessPacket callback
 *
 * @Return PVRSRV_ERROR error code, mainly:
 *         error coded returned by
 *           PVRSRVTLAcquireData()
 *           PVRSRVTLReleaseData()
 */

PVRSRV_ERROR PVRSRVDIListAllEntries(PVRSRV_DEV_CONNECTION *psConnection,
                                      DI_CONTEXT *psContext,
                                      PFN_PROCESS_PACKET pfnProcessPacket,
                                      void *pvPriv);

#endif /* PVRSRV_DI_H */
