/*************************************************************************/ /*!
@File
@Title          PVRSRV HWPerf Client API
@Copyright      Copyright (c) Imagination Technologies Ltd. All Rights Reserved
@Description    PVRSRV HWPerf Client functions
@License        Strictly Confidential.
*/ /**************************************************************************/

#ifndef RGXAPI_HWPERF_COMMON_H
#define RGXAPI_HWPERF_COMMON_H

#include "img_types.h"
#include "img_defs.h"
#include "pvrsrv_error.h"
#include "services.h"

#include "rgx_hwperf.h"
#include "rgx_hwperf_client.h"

#include "devicemem_typedefs.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*!
 ******************************************************************************
@mainpage Introduction and Document Scope


This document is a function reference for the Hardware Performance (HWPerf)
profiling and debug features of the PowerVR(r) Graphics GPU firmware and host
driver. It describes:

- The data made available via this feature
- The APIs available to access and control this data


<b>Overview</b>

 Hardware performance and debugging data is based on events emitted by various
 actors in the driver. The information about a single event is carried in a
 packet. There are three sources of events (or streams) available. These are:

- <b>Services Firmware</b>

  The Firmware uses a circular buffer to record information about a defined set of
  Firmware (FW) and hardware (HW) events. Each Packet carries information about
  the event and the state of different FW or HW units at the point at which the
  Packet was generated.
  The host reads those Packets and saves them in another circular buffer
  accessible through an API. If the host cannot keep up with the Firmware rate of
  producing packets, the Firmware discards new packets until there is space for
  them.
  Every packet contains a sequential number (ordinal) incremented for every event,
  regardless of the event being recorded or not (due to full buffers).

- <b>Services KM/Server module</b>

  The kernel driver running on the target host uses a circular buffer as a
  carrier for the data.
  When an event is generated a Packet is written into this buffer
  which can be read by a client. If the buffer becomes full and can't fit any
  more packets new packets will be discarded.
  Every packet contains a sequential number (ordinal) incremented for every
  event, regardless of the event being recorded or not (due to a full buffer).
  Any gap in sequence means that one or more packets were lost, indicating the
  information as incomplete.
  The sequential number starts from 0 (set on buffer creation) and increments
  with each event.

- <b>Services and Client Driver libraries (with client applications)</b>

  In a similar fashion to the kernel driver above, client/UM drivers running in
  user space applications can now generate events which can be captured along
  with the other sources from the PowerVR Graphics DDK in the same manner.


<b>Services Transport Layer UM Client API</b>

@par
This documents the interfaces available for the HWPerf data retrieval client
applications to use to obtain the HWPerf data generated by the client driver,
kernel server and firmware actors to their event streams.
See Section 2 and include/pvrsrv_tl.h for more information.


<b>HWPerf Client Event Format</b>

@par
The common HWPerf Client Event format, which is produced by the client driver
actors in the DDK, is available to all versions of the PowerVR Graphics DDK
and is documented in Section 3 and include/rgx_hwperf_client.h.


<b>HWPerf Client APIs</b>

@par
The common HWPerf Client APIs provide a way to control the HWPerf data generated
and are available to all versions of the PowerVR Graphics DDK and are documented
in Section 5 and include/rgxapi_hwperf_common.h.
The PowerVR Hardware specific APIs are documented in Section 7 and differ
between PowerVR(r) Rogue and PowerVR(r) Volcanic implementations. The Hardware
specific APIs are documented in include/rogue/rgxapi_hwperf.h (for PowerVR Rogue
architecture) and include/volcanic/rgxapi_hwperf.h (for PowerVR Volcanic
architecture).


<b>HWPerf Packet / Event Data Stream</b>

@par
This section describes the definitions used in the common parts (header etc.)
of the HWPerf event format as well as the event data from the kernel
server (host) and firmware actors in their associated event streams.
@par
An HWPerf event is made up of a sequence of paired data elements - an
event header and event payload. The header is defined in the C type
RGX_HWPERF_V2_PACKET_HDR and contains, amongst other things, the
overall size of the data element pair making up the event.
The payload element is of variable size
and is based on the C type RGX_HWPERF_V2_PACKET_DATA
The event definitions and structures are different for PowerVR(r) Rogue and
PowerVR(r) Volcanic GPUs. See Sections 4, 6 and include/rogue/rgx_hwperf.h or
include/volcanic/rgx_hwperf.h for more information.


<b>HWPerf Server/KM Kernel Driver APIs</b>

@par
The Server/KM Kernel Driver APIs are documented in Section 8 and detail the
routines available to be called from the Host driver DDK KM interface level to
control and retrieve the HWPerf data.
They vary depending on the PowerVR(r) hardware supported.
The routines are documented in
services/include/rogue/rgxapi_km.h and services/include/volcanic/rgxapi_km.h
respectively.


<b>HWPerf Structure Definitions</b>

@par
The structures specified within an HWPerf event stream are documented in
Section 9. This section describes the structures used in the events generated
by the three actors noted in the Overview above.

For more detailed information see the
<b>PowerVR Graphics DDK Services.Hardware Performance and Debug Logging</b> guide.
 *****************************************************************************/

/******************************************************************************
 * RGX HW Performance Profiling API(s)
 *****************************************************************************/

/**************************************************************************/ /*!
@Function     RGXGetHWPerfBvncFeatureFlags
@Description  Function returns the BVNC Feature flags of the connected device
               relevant to clients who process HWPerf event data.

@Input        psConnection           pointer to the opened connection to the
                                      driver
@Output       psBVNC                 Pointer (must not be null) which will be
                                      filled to return the feature information.
@Return                              PVRSRV_OK on success. Otherwise, a
                                      PVRSRV_* error code
*/ /***************************************************************************/
IMG_EXPORT PVRSRV_ERROR
RGXGetHWPerfBvncFeatureFlags(PVRSRV_DEV_CONNECTION *psConnection,
		                     RGX_HWPERF_BVNC       *psBVNC);

/**************************************************************************/ /*!
@Function       RGXCtrlHWPerf
@Description    Enable or disable the generation of RGX HWPerf event packets.
                 Depending on the stream id, behaviour for different streams may
                 be slightly different.
                 Enabling events for firmware buffer will reset the ordinal
                 and clear the event buffer used by the firmware. Events already
                 in the host driver buffer will be unaffected. Disabling events
                 simply stops event generation; events may continue to be be
                 drained from the buffers by the client.
                 When toggling it is recommended
                 to start from a defined and known state, e.g. all events were
                 disabled with a 'set' call before.
                 Enabling events for host buffer is simpler and just allows
                 events to be written to the buffer.
                 Disabling stops event emission to the buffer.
                 Enabling events for client buffers works like for host buffers.
                 The difference is with the passed parameters. For client
                 streams additional internal value is included in the ui64Mask
                 hence this function should not be called for client streams
                 directly.  Please use PVRSRVSetClientEventFilter() instead.

@Input          psDevConnection  pointer to the opened connection to the driver
@Input          eStreamID        ID of the stream to which the configuration
                                  applies
@Input          bToggle          Choose to 'toggle' bits in mask or 'set' whole
                                  mask and overwrite the old state.
                                  I.e. with toggle set we XOR in the mask bits,
                                  with toggle not set we assign the new mask.
@Input          ui64Mask         Mask of events to control. Each bit in the mask
                                  corresponds to one particular event type
                                  e.g. <tt>1U<<RGX_HWPERF_HW_TAKICK</tt>.
                                  Each bit set will lead to the event being
                                  enabled or disabled. Predefined masks like
                                  <tt>RGX_HWPERF_EVENT_MASK_ALL</tt> can be
                                  found in rgx_hwperf.h.
                                  Firmware stream mask occupies whole 64 bits of
                                  the passed value.
                                  Host stream mask takes 32 least significant
                                  bits and most significant bits are ignored.
                                  Client streams mask uses lower 32 bits to pass
                                  the mask value and upper 32 bits to pass
                                  internal Information Page index.
@Return                          PVRSRV_OK on success. Otherwise, a
                                  PVRSRV_* error code
*/ /***************************************************************************/
IMG_EXPORT
PVRSRV_ERROR RGXCtrlHWPerf(
		PVRSRV_DEV_CONNECTION* psDevConnection,
		RGX_HWPERF_STREAM_ID eStreamID,
		IMG_BOOL         bToggle,
		IMG_UINT64       ui64Mask);

/**************************************************************************/ /*!
@Function       RGXDisableHWPerfCounters
@Description    Disable the performance counter block for one or more
                 device layout modules. This will disable the counter block
                 potentially saving power.
                 The configuration of the counter block will be retained.

@Input          psDevConnection  pointer to the opened connection to the driver
@Input          ui32NumBlocks    Number of elements in the array
@Input          aui16BlockIDs    An array of words with values taken from
                                  the <tt>RGX_HWPERF_CNTBLK_ID</tt> enumeration.
@Return                          PVRSRV_OK on success. Otherwise, a
                                  PVRSRV_* error code
*/ /***************************************************************************/
IMG_EXPORT
PVRSRV_ERROR RGXDisableHWPerfCounters(
		PVRSRV_DEV_CONNECTION*  psDevConnection,
		IMG_UINT32        ui32NumBlocks,
		IMG_UINT16*       aui16BlockIDs);

/**************************************************************************/ /*!
@Function       RGXEnableHWPerfCounters
@Description    Enable the performance counter block for one or more
                 device layout modules. This will restore the configuration that
                 was previously set when the block was disabled.

@Input          psDevConnection  pointer to the opened connection to the driver
@Input          ui32NumBlocks    Number of elements in the array
@Input          aui16BlockIDs    An array of words with values taken from
                                  the <tt>RGX_HWPERF_CNTBLK_ID</tt> enumeration.
@Return                          PVRSRV_OK on success. Otherwise, a
                                  PVRSRV_* error code
*/ /***************************************************************************/
IMG_EXPORT
PVRSRV_ERROR RGXEnableHWPerfCounters(
		PVRSRV_DEV_CONNECTION*  psDevConnection,
		IMG_UINT32        ui32NumBlocks,
		IMG_UINT16*       aui16BlockIDs);

/**************************************************************************/ /*!
@Function       PVRSRVWriteClientEvent
@Description    Posts event to the client stream.

@Input          psDevConnection  pointer to the opened connection to the driver
@Input          eType            event's id
@Input          psPacketData     pointer to the packet's data
@Input          uiPacketDataSize         size of the psPacket data
@Return                          PVRSRV_OK on success. Otherwise, a
                                  PVRSRV_* error code
*/ /***************************************************************************/
IMG_EXPORT
PVRSRV_ERROR PVRSRVWriteClientEvent(PVRSRV_DEV_CONNECTION *psDevConnection,
                                 RGX_HWPERF_CLIENT_EVENT_TYPE eType,
                                 RGX_HWPERF_V2_PACKET_DATA_CLIENT *psPacketData,
                                 size_t uiPacketDataSize);

/**************************************************************************/ /*!
@Function       PVRSRVSetClientEventFilter
@Description    Sets event mask for the given API.

@Input          psDevConnection  pointer to the opened connection to the driver
@Input          eApi             API id that the filter will be set for
@Input          ui32Filter       events mask
@Return                          PVRSRV_OK on success. Otherwise, a
                                  PVRSRV_* error code
*/ /***************************************************************************/
IMG_EXPORT
PVRSRV_ERROR PVRSRVSetClientEventFilter(PVRSRV_DEV_CONNECTION *psDevConnection,
                                        RGX_HWPERF_CLIENT_API eApi,
                                        IMG_UINT32 ui32Filter);

/**************************************************************************/ /*!
@Function       PVRSRVGetClientEventFilter
@Description    Returns current event mask for the given API.

@Input          psDevConnection  pointer to the opened connection to the driver
@Input          eApi             API for which the filter will be retrieved
@Return                          value of the event's mask
*/ /***************************************************************************/
IMG_EXPORT
IMG_UINT32 PVRSRVGetClientEventFilter(PVRSRV_DEV_CONNECTION *psDevConnection,
                                      RGX_HWPERF_CLIENT_API eApi);

#if !defined(DOXYGEN)
/**************************************************************************/ /*!
@Function       PVRSRVRequestHWPerfResourceCapture
@Description    Request a device resource capture from a client specified by
                 PID and context ID.

@Input          psDevConnection      pointer to the opened connection to the driver
@Input          eResourceCaptureType The resource type to capture.
@Input          uPID                 The PID of the client process to capture from.
@Input          ui32ClientCtxID      The client context ID within the process to capture from.
@Input          ui32FrameNum         The frame number to (attempt) to capture.
@Input          fScale               The desired scaling factor to apply to the
                                      resource (if applicable) i.e. 1/2, 1/4,
                                      1/8 etc.
@Return                              PVRSRV_OK on success. Otherwise, a
                                      PVRSRV_* error code
*/ /***************************************************************************/
IMG_EXPORT
PVRSRV_ERROR PVRSRVRequestHWPerfResourceCapture(PVRSRV_DEV_CONNECTION *psDevConnection,
                                                RGX_HWPERF_RESOURCE_CAPTURE_TYPE eResourceCaptureType,
                                                IMG_PID uPID,
                                                IMG_UINT32 ui32ClientCtxID,
                                                IMG_UINT32 ui32FrameNum,
                                                IMG_FLOAT fScale);

/**************************************************************************/ /*!
@Function       PVRSRVGetHWPerfResourceCaptureResult
@Description    Retrieve the captured resource data, this function may be called
                 multiple times in order to receive all data transmitted by the
                 client process.

@Input          psDevConnection         pointer to the opened connection to the
                                         driver
@Input          ui32TimeoutMS           Time in milli-seconds to wait for the
                                         request to be completed.
@Input          ui32HeaderSizeBytes     Optionally create extra space in ppbData
                                         buffer to allow the caller to add a
                                         custom header to the data buffer.
                                         The resource data will be offset in
                                         ppbData by ui32HeaderSizeBytes.
@Input          psResourceCaptureResult Structure containing result details,
                                         fields are populated internally by the
                                         function.
@Return                                 RGX_HWPERF_RESOURCE_CAPTURE_RESULT_STATUS code.
*/ /***************************************************************************/
IMG_EXPORT
RGX_HWPERF_RESOURCE_CAPTURE_RESULT_STATUS PVRSRVGetHWPerfResourceCaptureResult(PVRSRV_DEV_CONNECTION *psDevConnection,
                                                                               IMG_UINT32 ui32TimeoutMS,
                                                                               IMG_UINT32 ui32HeaderSizeBytes,
                                                                               RGX_RESOURCE_CAPTURE_RESULT *psResourceCaptureResult);

/**************************************************************************/ /*!
@Function       PVRSRVSendHWPerfResourceRequestResult
@Description    Complete a resource request made by a HWPerf client, by making
                 the device buffer available to the requesting process.

@Input          psDevConnection  pointer to the opened connection to the driver
@Input          ui32CtxID        Callers context ID to query against
@Input          hMemDesc         Handle to the device memory descriptor where
                                  the requested resource data can be retrieved
                                  from.
                                  Note that the device memory should be
                                  allocated as exportable and CPU Readable.
                                  The caller should also ensure the meta-data
                                  defined in RGX_RESOURCE_CAPTURE_INFO
                                  is packed at the beginning of the device
                                  memory buffer.
@Input          eStatus          Pass <tt>RGX_HWPERF_RESOURCE_CAPTURE_RESULT_COMPLETE_SUCCESS</tt>
                                  or <tt>RGX_HWPERF_RESOURCE_CAPTURE_RESULT_COMPLETE_FAILURE</tt>
                                  to complete a request.
@Return                          PVRSRV_OK on success. Otherwise, a PVRSRV_*
                                  error code
*/ /***************************************************************************/
IMG_EXPORT
PVRSRV_ERROR PVRSRVSendHWPerfResourceRequestResult(PVRSRV_DEV_CONNECTION *psDevConnection,
                                                   IMG_UINT32 ui32CtxID,
                                                   DEVMEM_MEMDESC *hMemDesc,
                                                   RGX_HWPERF_RESOURCE_CAPTURE_RESULT_STATUS eStatus);

/**************************************************************************/ /*!
@Function       PVRSRVIsAnyHWPerfResourceCaptureRequest
@Description    Queries the info page to check whether there is a request for a
                 resource capture on this context.

@Input          psDevConnection pointer to the opened connection to the driver
@Input          ui32CtxID       Callers context ID to query against
@Input          ui32FrameNum    Callers current frame number
@Output         fScale          the desired scaling factor to apply to the
                                 resource (if applicable) i.e. 1/2, 1/4, 1/8
                                 etc.
@Return                         The resource capture type,
                                 <tt>RGX_HWPERF_RESOURCE_CAPTURE_TYPE_NONE</tt>
                                 if there is no capture request for this context
*/ /***************************************************************************/
IMG_EXPORT
RGX_HWPERF_RESOURCE_CAPTURE_TYPE PVRSRVIsAnyHWPerfResourceCaptureRequest(PVRSRV_DEV_CONNECTION *psDevConnection,
                                                                         IMG_UINT32 ui32CtxID,
                                                                         IMG_UINT32 ui32FrameNum,
                                                                         IMG_FLOAT *fScale);

#endif /* !defined(DOXYGEN) */
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* RGXAPI_HWPERF_COMMON_H */
