/*************************************************************************/ /*!
@File           rgxapi_tqtdm.h
@Title          TQ API Header
@Copyright      Copyright (c) Imagination Technologies Ltd. All Rights Reserved
@Description    Current blit programming model provides a flexible interface
                where the unit of operation is a blit, that contains a number of
                source surfaces - each can contain a number of sub rectangles
                and a single destination surface, where a corresponding sub
                rectangle is specified for each of the rectangles of the
                sources. Arbitrary scale and rotation are allowed per rectangle
                pairs. The blit also contains a scissor on the
                destination. Pixels outside of the scissor are not updated. The
                area inside the scissor but outside of the destination
                rectangles are either accumulated or filled with a fill colour.

                Sampler state or texture state changes require different TQ
                sources, even if they refer to the same surface in memory.

@License        Strictly Confidential.
*/ /**************************************************************************/

#ifndef TQ_TDM_API_H
#define TQ_TDM_API_H

#if defined(DOXYGEN)
#include "config.h"
#endif

#include "rgxdefs.h"
#include "img_types.h"
#include "img_defs.h"
#include "img_3dtypes.h"
#include "powervr/imgyuv.h"
#include "imgpixfmts_internal.h"
#include "rgx_common.h"
#include "pvrsrv_devmem.h"
#include "pvrsrv_sync_um.h"
#include "pvrsrv_devvar.h"

/*! ***********************************************************************//**
 TQ surface flags
*/ /**************************************************************************/
/* @{*/
#define TQ_SURFACE_FLAG_STRIDE_IN_BYTES             0x00000001U    /*!< stride is in bytes not texels (packed rgb888 format) */
#define TQ_SURFACE_FLAG_RB_SWAP                     0x00000002U    /*!< pixels have red and blue swapped */
#define TQ_SURFACE_FLAG_ENDIAN_SWAP                 0x00000004U    /*!< colour channels are endian swapped */
#define TQ_SURFACE_FLAG_BUFFER_SYNC                 0x00000008U    /*!< use buffer synchronisation */
#define TQ_SURFACE_FLAG_FBC_FLOAT                   0x00000010U    /*!< read/write compressed format as float */
#define TQ_SURFACE_FLAG_PICKD                       0x00000020U    /*!< depth channel operation*/
/* @}*/

/*! ************************************************************************//**
 @brief         A surface describes a source or destination for a transfer operation

 @note          As surfaces here are used in both source/destination context we
                refer to texels and pixels as they were the same thing.

				The address of the first PBE MRT is pointed to by sDevVAddr.
				With 2LEVELS flag set ui32ChunkStride is added to
				this address, to obtain the base address of the second PBE MRT.
*/ /**************************************************************************/
typedef struct TQ_SURFACE_TAG
{
	/*! Surface related flags TQ_SURFACE_FLAG_<*> */
	IMG_UINT32         ui32Flags;

	/*! Memory virtual address.
	 *
	 * Some address alignments may not be supported by HW. For FBC surfaces the
	 * address must point to either start of headers or start of data depending
	 * on HW. On HW using FBC descriptors, the address uses FBC descriptor
	 * indexing format.
	 */
	IMG_DEV_VIRTADDR   sDevVAddr;

	/*! Memory descriptor, read for buffer sync information. No mappings are read from this */
	PVRSRV_MEMDESC     hMemDesc;

	/*! Memory address for extra YUV planes. Same format as sDevVAddr. */
	IMG_DEV_VIRTADDR   asUVAddress[2];

	/*! Memory descriptors for extra YUV planes, read for buffer sync information, no mappings are read from this */
	PVRSRV_MEMDESC     ahUVMem[3];

	/*! YUV colour space. Must be specified for YUV pixel format. */
	IMG_YUV_COLORSPACE eYUVColourSpace;

	/*! surface width in texels */
	IMG_UINT32         ui32Width;

	/*! surface height in texels */
	IMG_UINT32         ui32Height;

	/*! surface depth. only applicable if IMG_MEMLAYOUT_3DTWIDDLED */
	IMG_UINT32         ui32Depth;

	/*! Z position in a 3D texture. 0.0f <= fZPosition <= ui32Depth
	 *
	 * The fractional part of this setting will only be used for source surfaces
	 * with Linear or Bilinear filtering enabled. Otherwise, it's safe to pass
	 * either the slice index or the midpoint of the slice, as the fractional
	 * part will have no effect.
	 */
	IMG_FLOAT          fZPosition;

	/*! stride in texels. Applies only to strided memory layout. */
	IMG_UINT32         ui32StrideInTexels;

	/*! pixel format*/
	IMG_PIXFMT         ePixFormat;

	/*! memory layout*/
	IMG_MEMLAYOUT      eMemLayout;

	/*! used with the 2LEVELS flag - see notes for details */
	IMG_UINT32         ui32ChunkStride;

	/*! the number of samples per texel/pixel (0 = 1, 1 = 1) */
	IMG_UINT32         ui32SampleCount;

	/*! Frame buffer compression */
	IMG_FB_COMPRESSION eFBCompression;
} TQ_SURFACE;


/*! ***********************************************************************//**
 @brief         Defines a mapping between source and destination sub rectangles

 Arbitrary coordinate flipping and rotation are supported.  Rotation refers to
 the sampler, the source and destination rectangles are specified in texel/pixel
 positions, then rotation rotates and scales the content inside the
 subrectangle.  Rotation does not affect where texels are read from and where
 they are written to.
*/ /**************************************************************************/
typedef struct TQ_RECTANGLE_MAPPING_TAG
{
	/*! rotation of the specified mapping*/
	IMG_ROTATION        eRotation;
	IMG_RECT            sSrcRectangle;
	IMG_RECT            sDstRectangle;
} TQ_RECTANGLE_MAPPING;


/*! ***********************************************************************//**
 @brief          Extends the TQ_SURFACE structure with input specific data
*/ /**************************************************************************/
typedef struct TQ_SOURCE_TAG
{
	/*! source surface */
	TQ_SURFACE              sSurface;

	/*! source addressing mode.
	 *
	 * In case of a simple 1:1 copy this setting does not affect the output but
	 * will affect performance. Use don't care or clamp when possible.
	 */
	IMG_ADDRESSMODE         eAddressMode;

	/*! source filtering method */
	IMG_FILTER              eFilter;

	/*! MSAA resolve operation */
	IMG_RESOLVE_OP          eResolveOp;

	/*! rectangle pairs of source -> destination mappings
	 */
	/* @{ */
	/*! number of mappings for the source */
	IMG_UINT32              ui32NumRectangleMappings;

	/*! mappings array */
	TQ_RECTANGLE_MAPPING  * pasRectangleMappings;
	/* @} */
} TQ_SOURCE;


/*! ***********************************************************************//**
 @brief          The transfer command characteristics
*/ /**************************************************************************/
#if defined(SUPPORT_WORKLOAD_ESTIMATION)
typedef struct RGX_TQ_WORKLOAD_CHARACTERISTICS_TAG
{
	IMG_UINT32	ui32DestSizeInTiles;
	IMG_UINT32	ui32PixFormatID;
} RGX_TQ_WORKLOAD_CHARACTERISTICS;
#endif

/*! TQ surfaces for rop4 blits
*/
/* @{*/
#define	TQ_MAX_SRC_SURFACES			3U	/*!< maximum number of source textures*/

#define TQ_SRC_SURFACE				0U	/*!< the surface where we read the source from*/
#define TQ_SRC_PATTERN				1U	/*!< the surface where we read the pattern from*/
#define TQ_SRC_MASK					2U	/*!< the surface where we read the mask from*/

#define	TQ_MAX_DST_SURFACES			1U	/*!< maximum number of destination textures*/

#if defined(RGX_FEATURE_PBE_STRIDE_ALIGN_1PIXEL)
#define TQ_MAX_STRIDE_IN_PIXELS_3D      16384U
#else
#define TQ_MAX_STRIDE_IN_PIXELS_3D      8192U
#endif

#define TQ_MAX_STRIDE_IN_PIXELS_TLA		16384U

#define TQ_MAX_PIXELS_VZ_TLA            0 /*!< TLA blit splitting disabled by default in virtualization builds */
#define TQ_MAX_PIXELS_VZ_3D             0 /*!< 3D TQ blit splitting disabled by default in virtualization builds */

/* @}*/


/*! Blitter priority selection
*/
/* @{*/
#define TQ_BLITTER_TLA			1U		/*!< blit routed to the TLA hardware*/
#define TQ_BLITTER_3D			2U		/*!< blit routed to the 3D hardware*/
#define TQ_BLITTER_TDM			4U		/*!< blit routed to the 3D hardware*/

/* default routing fallback list:
 *  - try TLA first
 *  - try 3D core second
 */
#define	TQ_ROUTE_AUTO			0x01020000U	/*!< left to right priority list of the blit routing*/
#define	TQ_ROUTE_ONLY_TLA		0x01000000U	/*!< TLA hardware only*/
#define	TQ_ROUTE_ONLY_3D		0x02000000U	/*!< 3D core only*/
#define TQ_ROUTE_PREFER_3D		0x02010000U	/*!< Prefer 3D core over TLA hardware */

/* @}*/

/**************************************************************************//**
@brief       TQ Alpha blend
******************************************************************************/
/*! Alpha blend type

  Alpha blending is supported on TLA and TQ3D.
*/
/* @{*/
#define TQ_ALPHA_NONE						0x00U		/*!< Alpha blend type 'none' */
#define TQ_ALPHA_SOURCE						0x01U		/*!< Cdst = Csrc*Asrc	+ Cdst*(1-Asrc)*/
#define TQ_ALPHA_PREMUL_SOURCE				0x02U		/*!< Cdst = Csrc		+ Cdst*(1-Asrc)*/
#define TQ_ALPHA_GLOBAL						0x03U		/*!< Cdst = Csrc*Aglob	+ Cdst*(1-Aglob)*/
#define TQ_ALPHA_PREMUL_SOURCE_WITH_GLOBAL	0x04U		/*!< Cdst = Csrc*Aglob	+ Cdst*(1-Asrc*Aglob)*/
#define TQ_ALPHA_CUSTOM						0x05U		/*!< programmable alpha and rgb blend ops */
#define TQ_ALPHA_AATEXT						0x06U		/*!< Cdst = Csrc*Asrc	+ Cdst*(1-Asrc) const rgb, A8 or A4 source */

/*
	TQ_ALPHA_CUSTOM : blend op flags for ui32CustomRGB and ui32CustomAlpha fields
	output = ([invert]source op * source rgb) + ([invert]dest op * dest rgb)
*/
#define TQ_ALPHA_CUSTOM_SRC_OP_ONE			0x00000000U  /*!< Source Op One */
#define TQ_ALPHA_CUSTOM_SRC_OP_SRC			0x00000001U  /*!< Source Op Source */
#define TQ_ALPHA_CUSTOM_SRC_OP_DST			0x00000002U  /*!< Source Op Dest */
#define TQ_ALPHA_CUSTOM_SRC_OP_SRCGLOB		0x00000003U  /*!< Source Op Source-Global */
#define TQ_ALPHA_CUSTOM_SRC_OP_DSTGLOB		0x00000004U  /*!< Source Op Dest-Global */
#define TQ_ALPHA_CUSTOM_SRC_OP_GLOB			0x00000005U  /*!< Source Op Global */
#define TQ_ALPHA_CUSTOM_SRC_OP_ZERO			0x00000006U  /*!< Source Op Zero */
#define TQ_ALPHA_CUSTOM_SRC_OP_INVERT		0x00000010U  /*!< Source Op Invert */
#define TQ_ALPHA_CUSTOM_DST_OP_ONE			0x00000000U  /*!< Dest Op One */
#define TQ_ALPHA_CUSTOM_DST_OP_SRC			0x00000100U  /*!< Dest Op Source */
#define TQ_ALPHA_CUSTOM_DST_OP_DST			0x00000200U  /*!< Dest Op Dest */
#define TQ_ALPHA_CUSTOM_DST_OP_SRCGLOB		0x00000300U  /*!< Dest Op Source-Global */
#define TQ_ALPHA_CUSTOM_DST_OP_DSTGLOB		0x00000400U  /*!< Dest Op Dest-Global */
#define TQ_ALPHA_CUSTOM_DST_OP_GLOB			0x00000500U  /*!< Dest Op Global */
#define TQ_ALPHA_CUSTOM_DST_OP_ZERO			0x00000600U  /*!< Dest Op Zero */
#define TQ_ALPHA_CUSTOM_DST_OP_INVERT		0x00001000U  /*!< Dest Op Invert */

/* @}*/

/*! Describes an Alpha-Transparency Configuration - for Transfer Queue Use. */
typedef struct TQ_ALPHA_TAG
{
	IMG_UINT32			ui32Type;			/*!< alpha type */
	IMG_UINT32			ui32Global;			/*!< global alpha value */

	IMG_UINT32			ui32CustomRGB;		/*!< custom blend op for rgb */
	IMG_UINT32			ui32CustomAlpha;	/*!< custom blend op for alpha */
	IMG_UINT32			ui32Global2;		/*!< custom global alpha value for alpha output */
	IMG_BOOL			bGlobSrcMul;		/*!< custom multiplication of global and source alpha */
	IMG_BOOL			bZeroSrcATrans;		/*!< custom zero source alpha transparency stage */

	IMG_BOOL			bAlphaComponents;	/*!< enable argb1555 alpha components */
	IMG_UINT32			ui32Component0;		/*!< source alpha value when argb1555 alpha bit is 0 */
	IMG_UINT32			ui32Component1;		/*!< source alpha value when argb1555 alpha bit is 1 */

} TQ_ALPHA; /*!< Typedef for struct _TQ_ALPHA_ */

/*! Provides info about a Pattern Rectangle - for Transfer Queue Use. */
typedef struct TQ_PATTERN_TAG
{
	IMG_UINT32			ui32XStart;			/*!< X offset to the first pixel to read within the pattern rectangle */
	IMG_UINT32			ui32YStart;			/*!< Y offset to the first pixel to read within the pattern rectangle */

} TQ_PATTERN; /*!< Typedef for struct _TQ_PATTERN_ */

/*! ***********************************************************************//**
@brief          Input specific to blit type TQ_BLIT

Depth stencil merge allows merging a depth only / stencil only or depth and stencil
texture against a depth and stencil texture. The following combinations are valid:

 * src0 : depth or depth & stencil, dest : depth & stencil - flags : DSMERGE | PICKD
   Takes the depth of the source and the stencil of the destination and combines the two interleaved.

 * src0 : stencil or depth & stencil, dest : depth & stencil - flags : DSMERGE
   Takes the stencil of the source and the depth of the destination and combines the two interleaved.
*/ /**************************************************************************/

/*!
  Parameters for a Transfer-Queue Blit Operation
*/
typedef struct TQ_BLIT_OP_TAG
{
	IMG_UINT32        ui32RopCode;                   /*!< 16 bit rop4 (ie two 8 bit rop3's)*/
	IMG_UINT32        ui32ColourMask;                /*!< colour key mask*/
	TQ_ALPHA          sAlpha;                        /*!< alpha blend*/
	TQ_PATTERN        sPattern;                      /*!< pattern offsets*/
} TQ_BLIT_OP; /*!< Convenience typedef for struct _TQ_BLIT_OP_ */


/*! ***********************************************************************//**
@brief          The submit structure.

Per submit client data. For the size of internal submit data refer to
 * TQ_CREATE_CONTEXT.pui32SubmitSizeInBytes.
*/ /**************************************************************************/
typedef struct TQ_TRANSFER_SUBMIT_TAG
{
	IMG_HANDLE          hTransferIntSubmit;          /*!< internal submit data handle*/
	IMG_UINT32          ui32ChosenBlitter;           /*!< filled in by prepare, the chosen blitter as in ui32Routing packed to the right (LSB) (client writes to are ignored by TQ) */

	PVRSRV_FENCE        iCheckFence; /*!< Fence sync checked before the commands in this submit, or PVRSRV_NO_FENCE */
} TQ_TRANSFER_SUBMIT;

/*!
 * @AddToGroup WorkloadProcessingTypes
 * @{
 */

/*! ***********************************************************************//*!
 @Name Transfer/TQ operation flags
 Flags supported by the transfer operation parameters/command  i.e. @Ref TQ_TRANSFER_COMMAND_TAG
 @{
*/ /**************************************************************************/
#define TQ_CMD_FLAGS_DONTSUBMIT       0x00000001U /*!< Prepare call doesn't submit. Not supported by TDM transfers. */
#define TQ_CMD_FLAGS_FIRSTINBATCH     0x00000002U /*!< first prepare call after a submit. Not supported by TDM Transfers. */
#define TQ_CMD_FLAGS_PDUMP_CONTINUOUS 0x00000004U /*!< PDUMP TQ op even if it's outside of capture range */
#define TQ_CMD_FLAGS_SECURE           0x00000008U /*!< whether or not to enable DRM security */
#define TQ_CMD_FLAGS_TLA_MERGE        0x00000010U /*!< when in batched mode, merge TLA commands */
#define TQ_CMD_FLAGS_SUBMITONLY       0x00000020U /*!< Submit operation without validation */
#define TQ_CMD_FLAGS_CPU_FLUSH_SOURCE 0x00000040U /*!< If the CPU reads from the TQ source surface, the cache must be flushed first. Only supported in TQ manager. */
#define TQ_CMD_FLAGS_SINGLE_CORE      0x00000080U /*!< Use single core in a multi core setup */
#define TQ_CMD_FLAGS_DSMERGE          0x00000200U /*!< merge a depth or stencil against a depth + stencil texture*/
#define TQ_CMD_FLAGS_PICKD            0x00000400U /*!< valid if doing a DS merge with depth + stencil to depth + stencil*/
#define TQ_CMD_FLAGS_FILL             0x00000800U /*!< destination pixels not covered by any of the destination rectangles but inside the scissor are filled with the clear colour */
#define TQ_CMD_FLAGS_DEST_COHERENT    0x00001000U /*!< destination surface coherent. Only on TDM transfers. */
#define TQ_CMD_FLAGS_SKIP_FLUSH_INVAL 0x00002000U /*!< skip cache flush/inval at the end of the blit. Only on TDM transfers.*/
#define TQ_CMD_FLAGS_FLUSHCACHE       0x00004000U /*!< Terminate and flush previously opened stream before this blit. Only on TDM transfers. */
#define TQ_CMD_FLAGS_TRP_PROTECTED    0x00008000U /*!< Safety critical protected content. */
#define TQ_CMD_FLAGS_ROPCODE          0x00010000U /*!< enable rop4 for dest/source/pattern/mask. Only on TLA transfers. */
#define TQ_CMD_FLAGS_SOURCE_CKEY      0x00020000U /*!< enable source colour key. Only on TLA transfers. */
#define TQ_CMD_FLAGS_DEST_CKEY        0x00040000U /*!< enable dest colour key. Only on TLA transfers. */
#define TQ_CMD_FLAGS_2LEVELS          0x00080000U /*!< write 2 levels at once of a mip chain - applicable on TQ3D routes only */
#define TQ_CMD_FLAGS_CKPASS           0x00100000U /*!< default colour key type is reject. Only on TLA transfers. */
#define TQ_CMD_FLAGS_FAST2D           0x00200000U /*!< if using TQ3D, route to fast2d. Not supported by TDM transfers. */
#define TQ_CMD_FLAGS_WGP              0x00400000U /*!< */
/*! @} End of TQ commmand flags */
/*! @} End of AddToGroup WorkloadProcessingTypes */

/**************************************************************************/ /*!
 * @InGroup        WorkloadProcessingTypes
 * @Brief Transfer operation parameters/command
 *
 * Used for passing parameters to the Transfer interface (i.e.
 * \ref RGXQueueTransfer)
 */ /**************************************************************************/
typedef struct TQ_TRANSFER_COMMAND_TAG
{
	IMG_UINT32                 ui32Flags;            /*!< TQ command/operation flags, i.e. TQ_CMD_FLAGS_* */
	IMG_UINT32                 ui32Frame;            /*!< Frame number */
	IMG_UINT32                 ui32NumSources;       /*!< Number of sources in the pasSources array */
	TQ_SOURCE                * pasSources;           /*!< Array of source surfaces*/
	IMG_CLEAR_COLOUR           uClearColour;         /*!< Clear colour */
	IMG_RECT                   sScissor;             /*!< A destination scissor rectangle */
	TQ_SURFACE                 sDest;                /*!< The destination surface*/
	IMG_UINT32                 ui32NumDevVars;       /*!< Number of DevVar updates */
	PVRSRV_DEV_VAR_UPDATE      asDevVarUpdates[PVRSRV_MAX_DEV_VARS];   /*!< DevVar update array */
	PVRSRV_FENCE               iCheckFence;                            /*!< Fence sync checked before this command, or PVRSRV_NO_FENCE */
	IMG_UINT32                 ui32JobRef;                             /*! Client job reference - used in debug for tracking submitted work */

#if defined(SUPPORT_WORKLOAD_ESTIMATION)
	/*! TDM workload characteristics - used to estimate GPU usage by future workloads */
	RGX_TQ_WORKLOAD_CHARACTERISTICS	sWorkloadCharacteristics;
#endif

	union {
		TQ_BLIT_OP             sBlit;                /*!< Blit Operation Description */
	} Details;

#if defined(PVRSRV_SUPPORT_LEGACY_TQ_UM)
	IMG_UINT32                 ui32Routing;          /*!< Blit routing, use TQ_ROUTE_* defines */
	TQ_TRANSFER_SUBMIT         sTransferSubmit;      /*!< Structure that holds the submit information */
#endif
} TQ_TRANSFER_COMMAND;


/*! ***********************************************************************//**
TQ mipgen flags
*/ /**************************************************************************/
/* @{ */
/*! PDUMP TQ op even if it's outside of capture range */
#define TQ_TDM_MIPGEN_FLAGS_PDUMP_CONTINUOUS 0x00000001U
/*! Disable 2 level optimisation if first level width or height uneven */
#define TQ_TDM_MIPGEN_FLAGS_STRICT_BILINEAR  0x00000002U
/*! First level from separate surface */
#define TQ_TDM_MIPGEN_FLAGS_SEPARATE_SOURCE  0x00000004U
/*! whether or not to enable DRM security */
#define TQ_TDM_MIPGEN_FLAGS_SECURE           0x00000008U
/*! submit operation without validation */
#define TQ_TDM_MIPGEN_FLAGS_SUBMITONLY       0x00000010U
/*! Safety critical protected content */
#define TQ_TDM_MIPGEN_FLAGS_TRP_PROTECTED    0x00000020U
/*! Use single core in a multi core setup */
#define TQ_TDM_MIPGEN_FLAGS_SINGLE_CORE      0x00000040U
/* @} */


/*! ***********************************************************************//**
 @brief          The mipgen command
*/ /**************************************************************************/
typedef struct TQ_TDM_MIPGEN_COMMAND_TAG
{
	/*! flags (TQ_MIPGEN_FLAGS_*) */
	IMG_UINT32                 ui32Flags;

	/*! frame number*/
	IMG_UINT32                 ui32Frame;

	/*! source surface,
	  used when flag TQ_TDM_MIPGEN_FLAGS_SEPARATE_SOURCE is set */
	TQ_SURFACE                 sSource;

	/*! mipmap surface */
	TQ_SURFACE                 sDest;

	/*! start generating from level */
	IMG_UINT32                 ui32BaseLevel;

	/*! number of levels to generate */
	IMG_UINT32                 ui32NumLevelsToGenerate;

	/*! Number of DevVar updates */
	IMG_UINT32                 ui32NumDevVars;

	/*! DevVar update array */
	PVRSRV_DEV_VAR_UPDATE      asDevVarUpdates[PVRSRV_MAX_DEV_VARS];

	/*!< Fence sync checked before this command, or PVRSRV_NO_FENCE */
	PVRSRV_FENCE               iCheckFence;

	/*!< client job reference - used in debug for tracking submitted work */
	IMG_UINT32                 ui32JobRef;

#if defined(SUPPORT_WORKLOAD_ESTIMATION)
	/*!< TDM workload characteristics - used to estimate GPU usage by future workloads */
	RGX_TQ_WORKLOAD_CHARACTERISTICS	sWorkloadCharacteristics;
#endif
} TQ_TDM_MIPGEN_COMMAND;


/*! ***********************************************************************//*!
 TQ submit flags
*/ /**************************************************************************/
/* @{ */
/*! Terminate and flush previously opened stream between each blit */
#define TQ_TDM_SUBMIT_FLAGS_FLUSHCACHE       0x00000001U
/*! Use single core in a multi core setup */
#define TQ_TDM_SUBMIT_FLAGS_SINGLE_CORE      0x00000002U
/* @} */


/*! ***********************************************************************//**
 @brief          The submit command
*/ /**************************************************************************/
typedef struct TQ_TDM_SUBMIT_TAG
{
	/*! flags (TQ_SUBMIT_FLAGS_*) */
	IMG_UINT32                 ui32Flags;

	/*! frame number*/
	IMG_UINT32                 ui32Frame;

	/*! number of prepares in the pasPrepares array */
	IMG_UINT32                 ui32NumPrepares;

	/*! array of prepares */
	IMG_HANDLE               * pahPrepares;

	/*!< Fence sync checked before this command, or PVRSRV_NO_CHECK_FENCE_REQUIRED */
	PVRSRV_FENCE               iCheckFence;

	/*!< client job reference - used in debug for tracking submitted work */
	IMG_UINT32                 ui32JobRef;
} TQ_TDM_SUBMIT;

/*! ***********************************************************************//**
 @brief          Transfer context type

 QUEUE: Used with transfer queue APIs (RGXQueueTransfer, ...)
 PREPARE: Used with prepare API calls (RGXTDMPrepare, ...).
 SUBMIT: For submitting work with RGXTDMSubmit.
*/ /**************************************************************************/
typedef enum TQ_TDM_CONTEXT_TYPE_TAG
{
	TQ_TDM_CONTEXT_QUEUE   = 0,
	TQ_TDM_CONTEXT_PREPARE = 1,
	TQ_TDM_CONTEXT_SUBMIT  = 2,
} TQ_TDM_CONTEXT_TYPE;

/**************************************************************************/ /*!
 * @InGroup WorkloadContexts
 * @Brief Parameters for making a Transfer Context (or TC) creation request
 */ /**************************************************************************/
typedef struct TQ_CREATE_CONTEXT_TAG
{
	/*! Context creation flags */
	IMG_UINT32          ui32ContextFlags;

	/*! Device memory context*/
	PVRSRV_DEVMEMCTX    hDevMemContext;

#if defined(PVRSRV_SUPPORT_LEGACY_TQ_UM)
	/*! TQ provides the internal submit size to the client*/
	IMG_UINT32			ui32SubmitSizeInBytes;
#endif

	/*! Context switch priority */
	IMG_UINT32          ui32Priority;

	/*! A 1 character tag describing client API requesting tdm context, this
		will have the DM's resource tag (ie. TDM) appended to it. Used in
		naming resources associated with this context, eg. Timeline. */
	IMG_CHAR            cClientAPI;

	/*! Transfer context type */
	TQ_TDM_CONTEXT_TYPE eType;

	/*! TQ shared memory. */
	IMG_HANDLE          hStaticMem;

	/*! Address for FW to signal a context reset */
	IMG_UINT64          ui64RobustnessAddress;
} TQ_CREATE_CONTEXT;

#if defined(RGX_FEATURE_FASTRENDER_DM)

IMG_EXPORT
PVRSRV_ERROR RGXTDMQueueTransfer(
	IMG_HANDLE                    hTransferContext,
	const TQ_TRANSFER_COMMAND     * psQueueTransfer,
	PVRSRV_FENCE                  * piUpdateFence,
	const IMG_CHAR                * pcszFenceName);

/**************************************************************************/ /*!
@Function       RGXTDMMipgen
@Description    Creates a mipmap.
@Input          hTransferContext        handle to a transfer context
@Input          psMipgenCommand         Mipmap gen command.
@Input          piUpdateFence           fence sync
@Input          pcszFenceName           fence sync name
@Return                                 PVRSRV_OK on success. Otherwise, a PVRSRV_
                                        error code
 */ /**************************************************************************/
IMG_EXPORT
PVRSRV_ERROR RGXTDMMipgen(
	IMG_HANDLE                   hTransferContext,
	const TQ_TDM_MIPGEN_COMMAND  * psMipgenCommand,
	PVRSRV_FENCE                 * piUpdateFence,
	const IMG_CHAR               * pcszFenceName);

/*************************************************************************/ /*!
@Function       RGXTDMPrepare
@Description    Create TDM prepare handle.
                Used to pre-allocate resources for a TQ command, without
				submitting work to HW or doing a kernel transition.
				The returned handle can be submitted any number times using
				RGXTDMSubmit. A prepare handle may hold many blits and kicks
				to HW.
				A prepare handle is destroyed using RGXTDMDestroyPrepare;
				This will free the allocated resources.
				The preapre handle must not be destroyed until all submitted
				blits are done in HW. Destroying before all blits are done
				can lead to unexpected results.
				RGXTDMPrepare ignores iCheckfence, DevVars and JobRef in the
				TQ command. DevVars and JobRef can be supplied when
				submitting a prepare handle.
@Input          hTransferContext  TDM transfer context. Must be created using
                                  type TQ_TDM_CONTEXT_PREPARE.
@Input          psCommand         TDM transfer command.
@Output         phPrepare         Pointer to a prepare handle.
@Return         PVRSRV_ERROR      PVRSRV_OK or error code.
*/ /**************************************************************************/
IMG_EXPORT
PVRSRV_ERROR RGXTDMPrepare(
	IMG_HANDLE                    hTransferContext,
	const TQ_TRANSFER_COMMAND     * psCommand,
	IMG_HANDLE                    * phPrepare);

/*************************************************************************/ /*!
@Function       RGXTDMMipgenPrepare
@Description    Create TDM Mipgen prepare handle.
@Input          hTransferContext  TDM transfer context.
@Input          psMipgenCommand   TDM mipgen command.
@Output         phPrepare         Pointer to prepare handle.
@Return         PVRSRV_ERROR      PVRSRV_OK or error code.
*/ /**************************************************************************/
IMG_EXPORT
PVRSRV_ERROR RGXTDMMipgenPrepare(
	IMG_HANDLE                    hTransferContext,
	const TQ_TDM_MIPGEN_COMMAND   * psMipgenCommand,
	IMG_HANDLE                    * phPrepare);

/*************************************************************************/ /*!
@Function       RGXTDMDestroyPrepare
@Description    Destroy TDM prepare handle. Free resources allocated
                for the prepare.
@Input          hPrepare          Prepare handle.
@Return         PVRSRV_ERROR      PVRSRV_OK or error code.
*/ /**************************************************************************/
IMG_EXPORT
PVRSRV_ERROR RGXTDMDestroyPrepare(IMG_HANDLE hPrepare);

/*************************************************************************/ /*!
@Function       RGXTDMSubmit
@Description    Submit TDM prepares.
@Input          hTransferContext  TDM transfer context.
@Input          psSubmit          TDM Submit command
@Output         piUpdateFence     Set to not NULL to request an update fence.
@Input          pcszFenceName     Name for update fence.
@Return         PVRSRV_ERROR      PVRSRV_OK or error code.
*/ /**************************************************************************/
IMG_EXPORT
PVRSRV_ERROR RGXTDMSubmit(
	IMG_HANDLE          hTransferContext,
	const TQ_TDM_SUBMIT * psSubmit,
	PVRSRV_FENCE        * piUpdateFence,
	const IMG_CHAR      * pcszFenceName);

/**************************************************************************/ /*!
@Function       RGXTDMMipgenValidate
@Description    Validate TQ Mipgen command
@Input          psMipgenCommand         Mipgen command.
@Return                                 IMG_TRUE on success.
 */ /**************************************************************************/
IMG_EXPORT
IMG_BOOL RGXTDMMipgenValidate(
	const TQ_TDM_MIPGEN_COMMAND * psMipgenCommand);

/*************************************************************************/ /*!
@Function       RGXTDMCreateStaticMem
@Description    Create shared read-only TQ memory, for re-use in more than
                one TQ context. Handle given by this function can be set
                in TQ_CREATE_CONTEXT when creating a new TQ context.
                Memory contexts must be same when creating TQ context.
@Input          psDevConnection   Device connection.
@Input          hDevMemContext    Memory context.
@Output         phStaticMem       Pointer to TQ shared memory handle.
@Return         PVRSRV_ERROR      PVRSRV_OK or error code.
*/ /**************************************************************************/
IMG_EXPORT
PVRSRV_ERROR RGXTDMCreateStaticMem(
	PVRSRV_DEV_CONNECTION       * psDevConnection,
	PVRSRV_DEVMEMCTX              hDevMemContext,
	IMG_HANDLE                  * phStaticMem);

/*************************************************************************/ /*!
@Function       RGXTDMDestroyStaticMem
@Description    Destroy shared read-only TQ memory created
                by RGXTDMCreateStaticMem.
@Input          psDevConnection   Device connection.
@Input          hStaticMem        Shared memory handle.
@Return         PVRSRV_ERROR      PVRSRV_OK or error code.
*/ /**************************************************************************/
IMG_EXPORT
PVRSRV_ERROR RGXTDMDestroyStaticMem(PVRSRV_DEV_CONNECTION * psDevConnection,
									IMG_HANDLE              hStaticMem);

/**************************************************************************/ /*!
@Function       RGXTDMCreateTransferContext / RGXTDMCreateTransferContextCCB
@Description    Creates a transfer context.
				RGXTDMCreateTransferContextCCB will use custom CCB size.
@Input          psDevConnection         Pointer to Device-Data structure
@Input          psCreateTransfer        Parameters required for creation of a
                                        new transfer context
@Input(for CCB) ui32CCBAllocSizeLog2    Optional custom CCB size.
                ui32CCBMaxAllocSizeLog2 Might be 0 for default size.
@Output         phTransferContext       On success, points to the newly-created
                                        transfer context.
@Return                                 PVRSRV_OK on success. Otherwise, a PVRSRV_
                                        error code
 */ /**************************************************************************/
IMG_EXPORT
PVRSRV_ERROR RGXTDMCreateTransferContext(
	PVRSRV_DEV_CONNECTION       * psDevConnection,
	const TQ_CREATE_CONTEXT     * psCreateTransfer,
	IMG_HANDLE                  * phTransferContext);
IMG_EXPORT
PVRSRV_ERROR RGXTDMCreateTransferContextCCB(
	PVRSRV_DEV_CONNECTION       * psDevConnection,
	const TQ_CREATE_CONTEXT     * psCreateTransfer,
	IMG_UINT32					  ui32CCBAllocSizeLog2,
	IMG_UINT32					  ui32CCBMaxAllocSizeLog2,
	IMG_HANDLE                  * phTransferContext);

/**************************************************************************/ /*!
@Function       RGXDestroyTransferContext
@Description    Destroy the specified transfer context
@Input          hTransferContext        handle to a transfer context
@Return                                 PVRSRV_OK on success. Otherwise, a PVRSRV_
                                        error code
 */ /**************************************************************************/
IMG_EXPORT
PVRSRV_ERROR RGXTDMDestroyTransferContext(IMG_HANDLE hTransferContext);


/**************************************************************************/ /*!
@Function       RGXSetTransferContextPriority
@Description    Destroy the specified transfer context
@Input          psDevData               Pointer to Device-Data structure
@Input          hContext                Handle to a transfer context
@Input          ui32Priority            Priority to set
@Return                                 PVRSRV_OK on success. Otherwise, a PVRSRV_
                                        error code
 */ /**************************************************************************/
IMG_EXPORT
PVRSRV_ERROR RGXTDMSetTransferContextPriority(
	PVRSRV_DEV_CONNECTION * psDevConnection,
	IMG_HANDLE            hContext,
	IMG_UINT32            ui32Priority);


/**************************************************************************/ /*!
@Function       RGXTDMSetTransferContextFlags
@Description    Set the flags (properties) for the specified transfer context
@Input          psDevConnection         Pointer to Device-Data structure
@Input          hContext                Handle to a transfer context
@Input          ui32ContextFlags        Flags to set
@Return                                 PVRSRV_OK on success. Otherwise, a PVRSRV_
                                        error code
 */ /**************************************************************************/
IMG_EXPORT
PVRSRV_ERROR RGXTDMSetTransferContextFlags(PVRSRV_DEV_CONNECTION * psDevConnection,
										   IMG_HANDLE            hContext,
										   IMG_UINT32            ui32ContextFlags);

#endif /* defined(RGX_FEATURE_FASTRENDER_DM) */

#if defined(PVRSRV_SUPPORT_LEGACY_TQ_UM)
/**************************************************************************/ /*!
@InGroup        WorkloadContexts
@Function       RGXCreateTransferContext / RGXCreateTransferContextCCB
@Description    Creates a transfer context.
				RGXCreateTransferContextCCB will use custom CCB size.
@Input          psDevConnection				Pointer to Device-Data structure
@Input          psCreateTransfer			Parameters required for creation of a
                                        	new transfer context
@Input(for CCB) ui32TQ2DCCBAllocSizeLog2	Optional custom CCB size.
                ui32TQ2DCCBMaxAllocSizeLog2	Might be 0 for default size.
				ui32TQ3DCCBAllocSizeLog2
				ui32TQ3DCCBMaxAllocSizeLog2
@Output         phTransferContext			On success, points to
                                    		the newly-created transfer context.
@Return                                 	PVRSRV_OK on success. Otherwise,
                                        	a PVRSRV_ error code
 */ /**************************************************************************/
IMG_EXPORT
PVRSRV_ERROR RGXCreateTransferContext(PVRSRV_DEV_CONNECTION * psDevConnection,
												   TQ_CREATE_CONTEXT * psCreateTransfer,
												   IMG_HANDLE * phTransferContext);
IMG_EXPORT
PVRSRV_ERROR RGXCreateTransferContextCCB(PVRSRV_DEV_CONNECTION          * psDevConnection,
													  TQ_CREATE_CONTEXT * psCreateTransfer,
													  IMG_UINT32		ui32TQ2DCCBAllocSizeLog2,
													  IMG_UINT32		ui32TQ2DCCBMaxAllocSizeLog2,
													  IMG_UINT32		ui32TQ3DCCBAllocSizeLog2,
													  IMG_UINT32		ui32TQ3DCCBMaxAllocSizeLog2,
													  IMG_HANDLE		* phTransferContext);

/**************************************************************************/ /*!
@InGroup        WorkloadContexts
@Function       RGXDestroyTransferContext
@Description    Destroy the specified transfer context
@Input          hTransferContext        handle to a transfer context
@Return                                 PVRSRV_OK on success. Otherwise, a PVRSRV_
                                        error code
 */ /**************************************************************************/
IMG_EXPORT
PVRSRV_ERROR RGXDestroyTransferContext(IMG_HANDLE hTransferContext);

/**************************************************************************/ /*!
@InGroup        WorkloadContexts
@Function       RGXSetTransferContextPriority
@Description    Set the priority for the specified transfer context
@Input          psDevConnection         Pointer to Device-Data structure
@Input          hContext                Handle to a transfer context
@Input          ui32Priority            Priority to set
@Return                                 PVRSRV_OK on success. Otherwise, a
                                        PVRSRV_ERROR code
 */ /**************************************************************************/
IMG_EXPORT
PVRSRV_ERROR RGXSetTransferContextPriority(PVRSRV_DEV_CONNECTION * psDevConnection,
										   IMG_HANDLE              hContext,
										   IMG_UINT32              ui32Priority);

/**************************************************************************/ /*!
@Function       RGXSetTransferContextFlags
@Description    Set the flags (properties) for the specified transfer context
@Input          psDevConnection         Pointer to Device-Data structure
@Input          hContext                Handle to a transfer context
@Input          ui32ContextFlags        Flags to set
@Return                                 PVRSRV_OK on success. Otherwise, a PVRSRV_
                                        error code
 */ /**************************************************************************/
IMG_EXPORT
PVRSRV_ERROR RGXSetTransferContextFlags(PVRSRV_DEV_CONNECTION * psDevConnection,
										IMG_HANDLE            hContext,
										IMG_UINT32            ui32ContextFlags);

/**************************************************************************/ /*!
@InGroup        WorkloadProcessingTypes
@Function       RGXQueueTransfer
@Description    Handles the queue transfer request

@Input          hTransferContext The context created by CreateTransferContext()
@Input          psQueueTransfer  The transfer operation supplied by the client
@Output         piUpdateFence    The returned update fence,
                                 or PVRSRV_NO_FENCE_PTR
@Input          pszFenceName     Name to assign to the created update fence,
                                 or PVRSRV_NO_FENCE_PTR
@Return                          PVRSRV_OK on success. Otherwise, a
                                 PVRSRV_ERROR code
 */ /**************************************************************************/
IMG_EXPORT
PVRSRV_ERROR RGXQueueTransfer(IMG_HANDLE            hTransferContext,
							  TQ_TRANSFER_COMMAND * psQueueTransfer,
							  PVRSRV_FENCE        * piUpdateFence,
							  const IMG_CHAR      * pszFenceName);

/**************************************************************************/ /*!
@InGroup        WorkloadProcessingTypes
@Function       RGXSubmitTransfer
@Description    Transfer UM/KM submission.

@Input          hTransferContext        handle to a transfer context
@Input          psSubmitTransfer        Parameters detailing the Transfer-Queue
                                        Submission
@Output         piUpdateFence           The returned update fence,
                                        or PVRSRV_NO_FENCE_PTR
@Input          pszFenceName            Name to assign to the created update fence,
                                        or PVRSRV_NO_FENCE_PTR
@Return                                 PVRSRV_OK on success. Otherwise, a PVRSRV_
                                        error code
 */ /**************************************************************************/
IMG_EXPORT
PVRSRV_ERROR RGXSubmitTransfer(IMG_HANDLE           hTransferContext,
							   TQ_TRANSFER_SUBMIT * psSubmitTransfer,
							   PVRSRV_FENCE       * piUpdateFence,
							   const IMG_CHAR     * pszFenceName);
#endif /* defined(PVRSRV_SUPPORT_LEGACY_TQ_UM) */

/**************************************************************************/ /*!
@Function       RGXQueueValidate
@Description    Validate a transfer operation

@Input          psQueueTransfer  The transfer command to be validated
@Return                          Bitfield of valid routes
                                 TQ_BLITTER_3D / TQ_BLITTER_TLA / TQ_BLITTER_TDM
 */ /**************************************************************************/
IMG_EXPORT
IMG_UINT32 RGXQueueValidate(const TQ_TRANSFER_COMMAND * psQueueTransfer);

#endif /* TQ_TDM_API_H */

/**************************************************************************/ /*!
End of file (tqtdmapi.h)
 */ /**************************************************************************/
